{% extends DYNAMICFORMS.field_base_template %}
{% block field_input %}
  <textarea id="{{ field.uuid }}"
            name="{{ field.name }}"
            class="ui-widget ui-corner-all dynamicforms-inputs"
            {% if style.placeholder %}placeholder="{{ style.placeholder }}"{% endif %}
            {% if style.rows %}rows="{{ style.rows }}"{% endif %}
            {% if field.read_only %}readonly{% endif %}
  >{% if field.value %}{{ field.value }}{% endif %}</textarea>
{% endblock %}
