{% load static %}
<link href="{% static "select2/select2-4.0.5/css/select2.min.css" %}" rel="stylesheet">
<link href="{% static DYNAMICFORMS.select2_bs_css %}" rel="stylesheet">
<script src="{% static "select2/select2-4.0.5/js/select2.min.js" %}"></script>
