{% load i18n %}
<div id="df-progress-bar-container" title="{% trans 'Performing operation...' %}">
  <div id="df-progress-comment"></div>
  <div id="df-progress-bar-indeterminate" class="indeterminate" style="display: none"></div>
  <div id="df-progress-bar-determinate" ></div>
</div>
