{% extends DYNAMICFORMS.field_base_template %}
{% load dynamicforms %}
{% block field_input %}
<fieldset>
  {% for nested_field in field %}
    {% if not nested_field.read_only %}
      {% render_field nested_field style=style %}
    {% endif %}
  {% endfor %}
</fieldset>
{% endblock %}